@@ -16,14 +16,14 @@
 libraries.
 """
 
-from typing import Any, Dict, NamedTuple
+from typing import Any, NamedTuple
 
 from beets import util
 
 
 class Node(NamedTuple):
-    files: Dict[str, Any]
-    dirs: Dict[str, Any]
+    files: dict[str, Any]
+    dirs: dict[str, Any]
 
 
 def _insert(node, path, itemid):
@@ -49,7 +49,7 @@ def libtree(lib):
     """
     root = Node({}, {})
     for item in lib.items():
-        dest = item.destination(fragment=True)
-        parts = util.components(dest)
+        dest = item.destination(relative_to_libdir=True)
+        parts = util.components(util.as_string(dest))
         _insert(root, parts, item.id)
     return root
