@@ -12,8 +12,8 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-"""Converts tracks or albums to external directory
-"""
+"""Converts tracks or albums to external directory"""
+
 import logging
 import os
 import shlex
@@ -22,12 +22,13 @@ import tempfile
 import threading
 from string import Template
 
+import mediafile
 from confuse import ConfigTypeError, Optional
 
 from beets import art, config, plugins, ui, util
 from beets.library import Item, parse_query_string
 from beets.plugins import BeetsPlugin
-from beets.util import arg_encoding, par_map
+from beets.util import par_map
 from beets.util.artresizer import ArtResizer
 from beets.util.m3u import M3UFile
 
@@ -63,9 +64,7 @@ def get_format(fmt=None):
         command = format_info["command"]
         extension = format_info.get("extension", fmt)
     except KeyError:
-        raise ui.UserError(
-            'convert: format {} needs the "command" field'.format(fmt)
-        )
+        raise ui.UserError(f'convert: format {fmt} needs the "command" field')
     except ConfigTypeError:
         command = config["convert"]["formats"][fmt].get(str)
         extension = fmt
@@ -76,8 +75,8 @@ def get_format(fmt=None):
         command = config["convert"]["command"].as_str()
     elif "opts" in keys:
         # Undocumented option for backwards compatibility with < 1.3.1.
-        command = "ffmpeg -i $source -y {} $dest".format(
-            config["convert"]["opts"].as_str()
+        command = (
+            f"ffmpeg -i $source -y {config['convert']['opts'].as_str()} $dest"
         )
     if "extension" in keys:
         extension = config["convert"]["extension"].as_str()
@@ -85,18 +84,23 @@ def get_format(fmt=None):
     return (command.encode("utf-8"), extension.encode("utf-8"))
 
 
+def in_no_convert(item: Item) -> bool:
+    no_convert_query = config["convert"]["no_convert"].as_str()
+
+    if no_convert_query:
+        query, _ = parse_query_string(no_convert_query, Item)
+        return query.match(item)
+    else:
+        return False
+
+
 def should_transcode(item, fmt):
     """Determine whether the item should be transcoded as part of
     conversion (i.e., its bitrate is high or it has the wrong format).
     """
-    no_convert_queries = config["convert"]["no_convert"].as_str_seq()
-    if no_convert_queries:
-        for query_string in no_convert_queries:
-            query, _ = parse_query_string(query_string, Item)
-            if query.match(item):
-                return False
-    if config["convert"]["never_convert_lossy_files"] and not (
-        item.format.lower() in LOSSLESS_FORMATS
+    if in_no_convert(item) or (
+        config["convert"]["never_convert_lossy_files"]
+        and item.format.lower() not in LOSSLESS_FORMATS
     ):
         return False
     maxbr = config["convert"]["max_bitrate"].get(Optional(int))
@@ -114,23 +118,30 @@ class ConvertPlugin(BeetsPlugin):
                 "pretend": False,
                 "link": False,
                 "hardlink": False,
-                "threads": util.cpu_count(),
+                "threads": os.cpu_count(),
                 "format": "mp3",
                 "id3v23": "inherit",
                 "formats": {
                     "aac": {
-                        "command": "ffmpeg -i $source -y -vn -acodec aac "
-                        "-aq 1 $dest",
+                        "command": (
+                            "ffmpeg -i $source -y -vn -acodec aac -aq 1 $dest"
+                        ),
                         "extension": "m4a",
                     },
                     "alac": {
-                        "command": "ffmpeg -i $source -y -vn -acodec alac $dest",
+                        "command": (
+                            "ffmpeg -i $source -y -vn -acodec alac $dest"
+                        ),
                         "extension": "m4a",
                     },
                     "flac": "ffmpeg -i $source -y -vn -acodec flac $dest",
                     "mp3": "ffmpeg -i $source -y -vn -aq 2 $dest",
-                    "opus": "ffmpeg -i $source -y -vn -acodec libopus -ab 96k $dest",
-                    "ogg": "ffmpeg -i $source -y -vn -acodec libvorbis -aq 3 $dest",
+                    "opus": (
+                        "ffmpeg -i $source -y -vn -acodec libopus -ab 96k $dest"
+                    ),
+                    "ogg": (
+                        "ffmpeg -i $source -y -vn -acodec libvorbis -aq 3 $dest"
+                    ),
                     "wma": "ffmpeg -i $source -y -vn -acodec wmav2 -vn $dest",
                 },
                 "max_bitrate": None,
@@ -165,16 +176,17 @@ class ConvertPlugin(BeetsPlugin):
             "--threads",
             action="store",
             type="int",
-            help="change the number of threads, \
-                              defaults to maximum available processors",
+            help=(
+                "change the number of threads, defaults to maximum available"
+                " processors"
+            ),
         )
         cmd.parser.add_option(
             "-k",
             "--keep-new",
             action="store_true",
             dest="keep_new",
-            help="keep only the converted \
-                              and move the old files",
+            help="keep only the converted and move the old files",
         )
         cmd.parser.add_option(
             "-d", "--dest", action="store", help="set the destination directory"
@@ -198,16 +210,16 @@ class ConvertPlugin(BeetsPlugin):
             "--link",
             action="store_true",
             dest="link",
-            help="symlink files that do not \
-                              need transcoding.",
+            help="symlink files that do not need transcoding.",
         )
         cmd.parser.add_option(
             "-H",
             "--hardlink",
             action="store_true",
             dest="hardlink",
-            help="hardlink files that do not \
-                              need transcoding. Overrides --link.",
+            help=(
+                "hardlink files that do not need transcoding. Overrides --link."
+            ),
         )
         cmd.parser.add_option(
             "-m",
@@ -276,9 +288,9 @@ class ConvertPlugin(BeetsPlugin):
         quiet = self.config["quiet"].get(bool)
 
         if not quiet and not pretend:
-            self._log.info("Encoding {0}", util.displayable_path(source))
+            self._log.info("Encoding {}", util.displayable_path(source))
 
-        command = command.decode(arg_encoding(), "surrogateescape")
+        command = os.fsdecode(command)
         source = os.fsdecode(source)
         dest = os.fsdecode(dest)
 
@@ -292,10 +304,10 @@ class ConvertPlugin(BeetsPlugin):
                     "dest": dest,
                 }
             )
-            encode_cmd.append(args[i].encode(util.arg_encoding()))
+            encode_cmd.append(os.fsdecode(args[i]))
 
         if pretend:
-            self._log.info("{0}", " ".join(ui.decargs(args)))
+            self._log.info("{}", " ".join(args))
             return
 
         try:
@@ -303,28 +315,25 @@ class ConvertPlugin(BeetsPlugin):
         except subprocess.CalledProcessError as exc:
             # Something went wrong (probably Ctrl+C), remove temporary files
             self._log.info(
-                "Encoding {0} failed. Cleaning up...",
+                "Encoding {} failed. Cleaning up...",
                 util.displayable_path(source),
             )
             self._log.debug(
-                "Command {0} exited with status {1}: {2}",
+                "Command {0} exited with status {1.returncode}: {1.output}",
                 args,
-                exc.returncode,
-                exc.output,
+                exc,
             )
             util.remove(dest)
             util.prune_dirs(os.path.dirname(dest))
             raise
         except OSError as exc:
             raise ui.UserError(
-                "convert: couldn't invoke '{}': {}".format(
-                    " ".join(ui.decargs(args)), exc
-                )
+                f"convert: couldn't invoke {' '.join(args)!r}: {exc}"
             )
 
         if not quiet and not pretend:
             self._log.info(
-                "Finished encoding {0}", util.displayable_path(source)
+                "Finished encoding {}", util.displayable_path(source)
             )
 
     def convert_item(
@@ -346,6 +355,15 @@ class ConvertPlugin(BeetsPlugin):
             item = yield (item, original, converted)
             dest = item.destination(basedir=dest_dir, path_formats=path_formats)
 
+            # Ensure that desired item is readable before processing it. Needed
+            # to avoid any side-effect of the conversion (linking, keep_new,
+            # refresh) if we already know that it will fail.
+            try:
+                mediafile.MediaFile(util.syspath(item.path))
+            except mediafile.UnreadableFileError as exc:
+                self._log.error("Could not open file to convert: {}", exc)
+                continue
+
             # When keeping the new file in the library, we first move the
             # current (pristine) file to the destination. We'll then copy it
             # back to its old path or transcode it to a new path.
@@ -369,21 +387,20 @@ class ConvertPlugin(BeetsPlugin):
 
             if os.path.exists(util.syspath(dest)):
                 self._log.info(
-                    "Skipping {0} (target file exists)",
-                    util.displayable_path(item.path),
+                    "Skipping {.filepath} (target file exists)", item
                 )
                 continue
 
             if keep_new:
                 if pretend:
                     self._log.info(
-                        "mv {0} {1}",
-                        util.displayable_path(item.path),
+                        "mv {.filepath} {}",
+                        item,
                         util.displayable_path(original),
                     )
                 else:
                     self._log.info(
-                        "Moving to {0}", util.displayable_path(original)
+                        "Moving to {}", util.displayable_path(original)
                     )
                     util.move(item.path, original)
 
@@ -399,10 +416,10 @@ class ConvertPlugin(BeetsPlugin):
                     msg = "ln" if hardlink else ("ln -s" if link else "cp")
 
                     self._log.info(
-                        "{2} {0} {1}",
+                        "{} {} {}",
+                        msg,
                         util.displayable_path(original),
                         util.displayable_path(converted),
-                        msg,
                     )
                 else:
                     # No transcoding necessary.
@@ -412,9 +429,7 @@ class ConvertPlugin(BeetsPlugin):
                         else ("Linking" if link else "Copying")
                     )
 
-                    self._log.info(
-                        "{1} {0}", util.displayable_path(item.path), msg
-                    )
+                    self._log.info("{} {.filepath}", msg, item)
 
                     if hardlink:
                         util.hardlink(original, converted)
@@ -445,8 +460,7 @@ class ConvertPlugin(BeetsPlugin):
                 if album and album.artpath:
                     maxwidth = self._get_art_resize(album.artpath)
                     self._log.debug(
-                        "embedding album art from {}",
-                        util.displayable_path(album.artpath),
+                        "embedding album art from {.art_filepath}", album
                     )
                     art.embed_item(
                         self._log,
@@ -504,8 +518,7 @@ class ConvertPlugin(BeetsPlugin):
 
         if os.path.exists(util.syspath(dest)):
             self._log.info(
-                "Skipping {0} (target file exists)",
-                util.displayable_path(album.artpath),
+                "Skipping {.art_filepath} (target file exists)", album
             )
             return
 
@@ -515,8 +528,8 @@ class ConvertPlugin(BeetsPlugin):
         # Either copy or resize (while copying) the image.
         if maxwidth is not None:
             self._log.info(
-                "Resizing cover art from {0} to {1}",
-                util.displayable_path(album.artpath),
+                "Resizing cover art from {.art_filepath} to {}",
+                album,
                 util.displayable_path(dest),
             )
             if not pretend:
@@ -526,10 +539,10 @@ class ConvertPlugin(BeetsPlugin):
                 msg = "ln" if hardlink else ("ln -s" if link else "cp")
 
                 self._log.info(
-                    "{2} {0} {1}",
-                    util.displayable_path(album.artpath),
-                    util.displayable_path(dest),
+                    "{} {.art_filepath} {}",
                     msg,
+                    album,
+                    util.displayable_path(dest),
                 )
             else:
                 msg = (
@@ -539,10 +552,10 @@ class ConvertPlugin(BeetsPlugin):
                 )
 
                 self._log.info(
-                    "{2} cover art from {0} to {1}",
-                    util.displayable_path(album.artpath),
-                    util.displayable_path(dest),
+                    "{} cover art from {.art_filepath} to {}",
                     msg,
+                    album,
+                    util.displayable_path(dest),
                 )
                 if hardlink:
                     util.hardlink(album.artpath, dest)
@@ -564,13 +577,13 @@ class ConvertPlugin(BeetsPlugin):
         ) = self._get_opts_and_config(opts)
 
         if opts.album:
-            albums = lib.albums(ui.decargs(args))
+            albums = lib.albums(args)
             items = [i for a in albums for i in a.items()]
             if not pretend:
                 for a in albums:
                     ui.print_(format(a, ""))
         else:
-            items = list(lib.items(ui.decargs(args)))
+            items = list(lib.items(args))
             if not pretend:
                 for i in items:
                     ui.print_(format(i, ""))
@@ -603,19 +616,13 @@ class ConvertPlugin(BeetsPlugin):
             # Playlist paths are understood as relative to the dest directory.
             pl_normpath = util.normpath(playlist)
             pl_dir = os.path.dirname(pl_normpath)
-            self._log.info("Creating playlist file {0}", pl_normpath)
+            self._log.info("Creating playlist file {}", pl_normpath)
             # Generates a list of paths to media files, ensures the paths are
             # relative to the playlist's location and translates the unicode
             # strings we get from item.destination to bytes.
             items_paths = [
                 os.path.relpath(
-                    util.bytestring_path(
-                        item.destination(
-                            basedir=dest,
-                            path_formats=path_formats,
-                            fragment=False,
-                        )
-                    ),
+                    item.destination(basedir=dest, path_formats=path_formats),
                     pl_dir,
                 )
                 for item in items
@@ -636,8 +643,8 @@ class ConvertPlugin(BeetsPlugin):
             # Create a temporary file for the conversion.
             tmpdir = self.config["tmpdir"].get()
             if tmpdir:
-                tmpdir = util.py3_path(util.bytestring_path(tmpdir))
-            fd, dest = tempfile.mkstemp(util.py3_path(b"." + ext), dir=tmpdir)
+                tmpdir = os.fsdecode(util.bytestring_path(tmpdir))
+            fd, dest = tempfile.mkstemp(f".{os.fsdecode(ext)}", dir=tmpdir)
             os.close(fd)
             dest = util.bytestring_path(dest)
             _temp_files.append(dest)  # Delete the transcode later.
@@ -659,7 +666,7 @@ class ConvertPlugin(BeetsPlugin):
             if self.config["delete_originals"]:
                 self._log.log(
                     logging.DEBUG if self.config["quiet"] else logging.INFO,
-                    "Removing original file {0}",
+                    "Removing original file {}",
                     source_path,
                 )
                 util.remove(source_path, False)
