@@ -12,16 +12,21 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-"""Moves "featured" artists to the title from the artist field.
-"""
+"""Moves "featured" artists to the title from the artist field."""
+
+from __future__ import annotations
 
 import re
+from typing import TYPE_CHECKING
 
 from beets import plugins, ui
-from beets.util import displayable_path
+
+if TYPE_CHECKING:
+    from beets.importer import ImportSession, ImportTask
+    from beets.library import Item
 
 
-def split_on_feat(artist):
+def split_on_feat(artist: str) -> tuple[str, str | None]:
     """Given an artist string, split the "main" artist from any artist
     on the right-hand side of a string like "feat". Return the main
     artist, which is always a string, and the featuring artist, which
@@ -29,19 +34,26 @@ def split_on_feat(artist):
     """
     # split on the first "feat".
     regex = re.compile(plugins.feat_tokens(), re.IGNORECASE)
-    parts = [s.strip() for s in regex.split(artist, 1)]
+    parts = tuple(s.strip() for s in regex.split(artist, 1))
     if len(parts) == 1:
         return parts[0], None
     else:
-        return tuple(parts)
+        assert len(parts) == 2  # help mypy out
+        return parts
 
 
-def contains_feat(title):
+def contains_feat(title: str) -> bool:
     """Determine whether the title contains a "featured" marker."""
-    return bool(re.search(plugins.feat_tokens(), title, flags=re.IGNORECASE))
+    return bool(
+        re.search(
+            plugins.feat_tokens(for_artist=False),
+            title,
+            flags=re.IGNORECASE,
+        )
+    )
 
 
-def find_feat_part(artist, albumartist):
+def find_feat_part(artist: str, albumartist: str) -> str | None:
     """Attempt to find featured artists in the item's artist fields and
     return the results. Returns None if no featured artist found.
     """
@@ -70,14 +82,15 @@ def find_feat_part(artist, albumartist):
 
 
 class FtInTitlePlugin(plugins.BeetsPlugin):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
         self.config.add(
             {
                 "auto": True,
                 "drop": False,
-                "format": "feat. {0}",
+                "format": "feat. {}",
+                "keep_in_artist": False,
             }
         )
 
@@ -97,38 +110,52 @@ class FtInTitlePlugin(plugins.BeetsPlugin):
         if self.config["auto"]:
             self.import_stages = [self.imported]
 
-    def commands(self):
+    def commands(self) -> list[ui.Subcommand]:
         def func(lib, opts, args):
             self.config.set_args(opts)
             drop_feat = self.config["drop"].get(bool)
+            keep_in_artist_field = self.config["keep_in_artist"].get(bool)
             write = ui.should_write()
 
-            for item in lib.items(ui.decargs(args)):
-                self.ft_in_title(item, drop_feat)
-                item.store()
-                if write:
-                    item.try_write()
+            for item in lib.items(args):
+                if self.ft_in_title(item, drop_feat, keep_in_artist_field):
+                    item.store()
+                    if write:
+                        item.try_write()
 
         self._command.func = func
         return [self._command]
 
-    def imported(self, session, task):
+    def imported(self, session: ImportSession, task: ImportTask) -> None:
         """Import hook for moving featuring artist automatically."""
         drop_feat = self.config["drop"].get(bool)
+        keep_in_artist_field = self.config["keep_in_artist"].get(bool)
 
         for item in task.imported_items():
-            self.ft_in_title(item, drop_feat)
-            item.store()
+            if self.ft_in_title(item, drop_feat, keep_in_artist_field):
+                item.store()
 
-    def update_metadata(self, item, feat_part, drop_feat):
+    def update_metadata(
+        self,
+        item: Item,
+        feat_part: str,
+        drop_feat: bool,
+        keep_in_artist_field: bool,
+    ) -> None:
         """Choose how to add new artists to the title and set the new
         metadata. Also, print out messages about any changes that are made.
         If `drop_feat` is set, then do not add the artist to the title; just
         remove it from the artist field.
         """
-        # In all cases, update the artist fields.
-        self._log.info("artist: {0} -> {1}", item.artist, item.albumartist)
-        item.artist = item.albumartist
+        # In case the artist is kept, do not update the artist fields.
+        if keep_in_artist_field:
+            self._log.info(
+                "artist: {.artist} (Not changing due to keep_in_artist)", item
+            )
+        else:
+            self._log.info("artist: {0.artist} -> {0.albumartist}", item)
+            item.artist = item.albumartist
+
         if item.artist_sort:
             # Just strip the featured artist from the sort name.
             item.artist_sort, _ = split_on_feat(item.artist_sort)
@@ -139,12 +166,20 @@ class FtInTitlePlugin(plugins.BeetsPlugin):
             feat_format = self.config["format"].as_str()
             new_format = feat_format.format(feat_part)
             new_title = f"{item.title} {new_format}"
-            self._log.info("title: {0} -> {1}", item.title, new_title)
+            self._log.info("title: {.title} -> {}", item, new_title)
             item.title = new_title
 
-    def ft_in_title(self, item, drop_feat):
+    def ft_in_title(
+        self,
+        item: Item,
+        drop_feat: bool,
+        keep_in_artist_field: bool,
+    ) -> bool:
         """Look for featured artists in the item's artist fields and move
         them to the title.
+
+        Returns:
+            True if the item has been modified. False otherwise.
         """
         artist = item.artist.strip()
         albumartist = item.albumartist.strip()
@@ -152,17 +187,22 @@ class FtInTitlePlugin(plugins.BeetsPlugin):
         # Check whether there is a featured artist on this track and the
         # artist field does not exactly match the album artist field. In
         # that case, we attempt to move the featured artist to the title.
+        if not albumartist or albumartist == artist:
+            return False
+
         _, featured = split_on_feat(artist)
-        if featured and albumartist != artist and albumartist:
-            self._log.info("{}", displayable_path(item.path))
+        if not featured:
+            return False
 
-            feat_part = None
+        self._log.info("{.filepath}", item)
 
-            # Attempt to find the featured artist.
-            feat_part = find_feat_part(artist, albumartist)
+        # Attempt to find the featured artist.
+        feat_part = find_feat_part(artist, albumartist)
 
-            # If we have a featuring artist, move it to the title.
-            if feat_part:
-                self.update_metadata(item, feat_part, drop_feat)
-            else:
-                self._log.info("no featuring artists found")
+        if not feat_part:
+            self._log.info("no featuring artists found")
+            return False
+
+        # If we have a featuring artist, move it to the title.
+        self.update_metadata(item, feat_part, drop_feat, keep_in_artist_field)
+        return True
