@@ -21,15 +21,18 @@ and has been edited to remove some questionable entries.
 The scraper script used is available here:
 https://gist.github.com/1241307
 """
-import codecs
+
 import os
 import traceback
+from pathlib import Path
+from typing import Union
 
 import pylast
 import yaml
 
 from beets import config, library, plugins, ui
-from beets.util import normpath, plurality
+from beets.library import Album, Item
+from beets.util import plurality, unique_list
 
 LASTFM = pylast.LastFMNetwork(api_key=plugins.LASTFM_KEY)
 
@@ -39,16 +42,6 @@ PYLAST_EXCEPTIONS = (
     pylast.NetworkError,
 )
 
-REPLACE = {
-    "\u2010": "-",
-}
-
-
-def deduplicate(seq):
-    """Remove duplicates from sequence while preserving order."""
-    seen = set()
-    return [x for x in seq if x not in seen and not seen.add(x)]
-
 
 # Canonicalization tree processing.
 
@@ -101,14 +94,15 @@ class LastGenrePlugin(plugins.BeetsPlugin):
                 "fallback": None,
                 "canonical": False,
                 "source": "album",
-                "force": True,
+                "force": False,
+                "keep_existing": False,
                 "auto": True,
                 "separator": ", ",
                 "prefer_specific": False,
                 "title_case": True,
+                "extended_debug": False,
             }
         )
-
         self.setup()
 
     def setup(self):
@@ -117,52 +111,65 @@ class LastGenrePlugin(plugins.BeetsPlugin):
             self.import_stages = [self.imported]
 
         self._genre_cache = {}
+        self.whitelist = self._load_whitelist()
+        self.c14n_branches, self.canonicalize = self._load_c14n_tree()
 
-        # Read the whitelist file if enabled.
-        self.whitelist = set()
+    def _load_whitelist(self) -> set[str]:
+        """Load the whitelist from a text file.
+
+        Default whitelist is used if config is True, empty string or set to "nothing".
+        """
+        whitelist = set()
         wl_filename = self.config["whitelist"].get()
-        if wl_filename in (True, ""):  # Indicates the default whitelist.
+        if wl_filename in (True, "", None):  # Indicates the default whitelist.
             wl_filename = WHITELIST
         if wl_filename:
-            wl_filename = normpath(wl_filename)
-            with open(wl_filename, "rb") as f:
-                for line in f:
-                    line = line.decode("utf-8").strip().lower()
-                    if line and not line.startswith("#"):
-                        self.whitelist.add(line)
+            self._log.debug("Loading whitelist {}", wl_filename)
+            text = Path(wl_filename).expanduser().read_text(encoding="utf-8")
+            for line in text.splitlines():
+                if (line := line.strip().lower()) and not line.startswith("#"):
+                    whitelist.add(line)
 
-        # Read the genres tree for canonicalization if enabled.
-        self.c14n_branches = []
+        return whitelist
+
+    def _load_c14n_tree(self) -> tuple[list[list[str]], bool]:
+        """Load the canonicalization tree from a YAML file.
+
+        Default tree is used if config is True, empty string, set to "nothing"
+        or if prefer_specific is enabled.
+        """
+        c14n_branches: list[list[str]] = []
         c14n_filename = self.config["canonical"].get()
-        self.canonicalize = c14n_filename is not False
-
+        canonicalize = c14n_filename is not False
         # Default tree
-        if c14n_filename in (True, ""):
-            c14n_filename = C14N_TREE
-        elif not self.canonicalize and self.config["prefer_specific"].get():
+        if c14n_filename in (True, "", None) or (
             # prefer_specific requires a tree, load default tree
+            not canonicalize and self.config["prefer_specific"].get()
+        ):
             c14n_filename = C14N_TREE
-
         # Read the tree
         if c14n_filename:
-            self._log.debug("Loading canonicalization tree {0}", c14n_filename)
-            c14n_filename = normpath(c14n_filename)
-            with codecs.open(c14n_filename, "r", encoding="utf-8") as f:
+            self._log.debug("Loading canonicalization tree {}", c14n_filename)
+            with Path(c14n_filename).expanduser().open(encoding="utf-8") as f:
                 genres_tree = yaml.safe_load(f)
-            flatten_tree(genres_tree, [], self.c14n_branches)
+            flatten_tree(genres_tree, [], c14n_branches)
+        return c14n_branches, canonicalize
 
     @property
-    def sources(self):
+    def sources(self) -> tuple[str, ...]:
         """A tuple of allowed genre sources. May contain 'track',
         'album', or 'artist.'
         """
         source = self.config["source"].as_choice(("track", "album", "artist"))
         if source == "track":
             return "track", "album", "artist"
-        elif source == "album":
+        if source == "album":
             return "album", "artist"
-        elif source == "artist":
+        if source == "artist":
             return ("artist",)
+        return tuple()
+
+    # More canonicalization and general helpers.
 
     def _get_depth(self, tag):
         """Find the depth of a tag in the genres tree."""
@@ -182,14 +189,29 @@ class LastGenrePlugin(plugins.BeetsPlugin):
         depth_tag_pairs.sort(reverse=True)
         return [p[1] for p in depth_tag_pairs]
 
-    def _resolve_genres(self, tags):
-        """Given a list of strings, return a genre by joining them into a
-        single string and (optionally) canonicalizing each.
+    def _resolve_genres(self, tags: list[str]) -> list[str]:
+        """Canonicalize, sort and filter a list of genres.
+
+        - Returns an empty list if the input tags list is empty.
+        - If canonicalization is enabled, it extends the list by incorporating
+          parent genres from the canonicalization tree. When a whitelist is set,
+          only parent tags that pass a validity check (_is_valid) are included;
+          otherwise, it adds the oldest ancestor. Adding parent tags is stopped
+          when the count of tags reaches the configured limit (count).
+        - The tags list is then deduplicated to ensure only unique genres are
+          retained.
+        - If the 'prefer_specific' configuration is enabled, the list is sorted
+          by the specificity (depth in the canonicalization tree) of the genres.
+        - Finally applies whitelist filtering to ensure that only valid
+          genres are kept. (This may result in no genres at all being retained).
+        - Returns the filtered list of genres, limited to the configured count.
         """
         if not tags:
-            return None
+            return []
 
         count = self.config["count"].get(int)
+
+        # Canonicalization (if enabled)
         if self.canonicalize:
             # Extend the list to consider tags parents in the c14n tree
             tags_all = []
@@ -200,7 +222,7 @@ class LastGenrePlugin(plugins.BeetsPlugin):
                     parents = [
                         x
                         for x in find_parents(tag, self.c14n_branches)
-                        if self._is_allowed(x)
+                        if self._is_valid(x)
                     ]
                 else:
                     parents = [find_parents(tag, self.c14n_branches)[-1]]
@@ -215,7 +237,7 @@ class LastGenrePlugin(plugins.BeetsPlugin):
                     break
             tags = tags_all
 
-        tags = deduplicate(tags)
+        tags = unique_list(tags)
 
         # Sort the tags by specificity.
         if self.config["prefer_specific"]:
@@ -223,120 +245,179 @@ class LastGenrePlugin(plugins.BeetsPlugin):
 
         # c14n only adds allowed genres but we may have had forbidden genres in
         # the original tags list
-        tags = [self._format_tag(x) for x in tags if self._is_allowed(x)]
-
-        return (
-            self.config["separator"]
-            .as_str()
-            .join(tags[: self.config["count"].get(int)])
-        )
-
-    def _format_tag(self, tag):
-        if self.config["title_case"]:
-            return tag.title()
-        return tag
+        valid_tags = [t for t in tags if self._is_valid(t)]
+        return valid_tags[:count]
 
     def fetch_genre(self, lastfm_obj):
         """Return the genre for a pylast entity or None if no suitable genre
         can be found. Ex. 'Electronic, House, Dance'
         """
         min_weight = self.config["min_weight"].get(int)
-        return self._resolve_genres(self._tags_for(lastfm_obj, min_weight))
+        return self._tags_for(lastfm_obj, min_weight)
 
-    def _is_allowed(self, genre):
-        """Determine whether the genre is present in the whitelist,
-        returning a boolean.
+    def _is_valid(self, genre: str) -> bool:
+        """Check if the genre is valid.
+
+        Depending on the whitelist property, valid means a genre is in the
+        whitelist or any genre is allowed.
         """
-        if genre is None:
-            return False
-        if not self.whitelist or genre in self.whitelist:
+        if genre and (not self.whitelist or genre.lower() in self.whitelist):
             return True
         return False
 
-    # Cached entity lookups.
+    # Cached last.fm entity lookups.
 
     def _last_lookup(self, entity, method, *args):
         """Get a genre based on the named entity using the callable `method`
         whose arguments are given in the sequence `args`. The genre lookup
-        is cached based on the entity name and the arguments. Before the
-        lookup, each argument is has some Unicode characters replaced with
-        rough ASCII equivalents in order to return better results from the
-        Last.fm database.
+        is cached based on the entity name and the arguments.
+
+        Before the lookup, each argument has the "-" Unicode character replaced
+        with its rough ASCII equivalents in order to return better results from
+        the Last.fm database.
         """
         # Shortcut if we're missing metadata.
         if any(not s for s in args):
-            return None
+            return []
 
-        key = "{}.{}".format(entity, "-".join(str(a) for a in args))
-        if key in self._genre_cache:
-            return self._genre_cache[key]
-        else:
-            args_replaced = []
-            for arg in args:
-                for k, v in REPLACE.items():
-                    arg = arg.replace(k, v)
-                args_replaced.append(arg)
+        key = f"{entity}.{'-'.join(str(a) for a in args)}"
+        if key not in self._genre_cache:
+            args = [a.replace("\u2010", "-") for a in args]
+            self._genre_cache[key] = self.fetch_genre(method(*args))
 
-            genre = self.fetch_genre(method(*args_replaced))
-            self._genre_cache[key] = genre
-            return genre
+        genre = self._genre_cache[key]
+        if self.config["extended_debug"]:
+            self._log.debug("last.fm (unfiltered) {} tags: {}", entity, genre)
+        return genre
 
     def fetch_album_genre(self, obj):
-        """Return the album genre for this Item or Album."""
+        """Return raw album genres from Last.fm for this Item or Album."""
         return self._last_lookup(
             "album", LASTFM.get_album, obj.albumartist, obj.album
         )
 
     def fetch_album_artist_genre(self, obj):
-        """Return the album artist genre for this Item or Album."""
+        """Return raw album artist genres from Last.fm for this Item or Album."""
         return self._last_lookup("artist", LASTFM.get_artist, obj.albumartist)
 
     def fetch_artist_genre(self, item):
-        """Returns the track artist genre for this Item."""
+        """Returns raw track artist genres from Last.fm for this Item."""
         return self._last_lookup("artist", LASTFM.get_artist, item.artist)
 
     def fetch_track_genre(self, obj):
-        """Returns the track genre for this Item."""
+        """Returns raw track genres from Last.fm for this Item."""
         return self._last_lookup(
             "track", LASTFM.get_track, obj.artist, obj.title
         )
 
-    def _get_genre(self, obj):
-        """Get the genre string for an Album or Item object based on
-        self.sources. Return a `(genre, source)` pair. The
-        prioritization order is:
+    # Main processing: _get_genre() and helpers.
+
+    def _format_and_stringify(self, tags: list[str]) -> str:
+        """Format to title_case if configured and return as delimited string."""
+        if self.config["title_case"]:
+            formatted = [tag.title() for tag in tags]
+        else:
+            formatted = tags
+
+        return self.config["separator"].as_str().join(formatted)
+
+    def _get_existing_genres(self, obj: Union[Album, Item]) -> list[str]:
+        """Return a list of genres for this Item or Album. Empty string genres
+        are removed."""
+        separator = self.config["separator"].get()
+        if isinstance(obj, library.Item):
+            item_genre = obj.get("genre", with_album=False).split(separator)
+        else:
+            item_genre = obj.get("genre").split(separator)
+
+        # Filter out empty strings
+        return [g for g in item_genre if g]
+
+    def _combine_resolve_and_log(
+        self, old: list[str], new: list[str]
+    ) -> list[str]:
+        """Combine old and new genres and process via _resolve_genres."""
+        self._log.debug("raw last.fm tags: {}", new)
+        self._log.debug("existing genres taken into account: {}", old)
+        combined = old + new
+        return self._resolve_genres(combined)
+
+    def _get_genre(
+        self, obj: Union[Album, Item]
+    ) -> tuple[Union[str, None], ...]:
+        """Get the final genre string for an Album or Item object.
+
+        `self.sources` specifies allowed genre sources. Starting with the first
+        source in this tuple, the following stages run through until a genre is
+        found or no options are left:
             - track (for Items only)
             - album
-            - artist
-            - original
-            - fallback
+            - artist, albumartist or "most popular track genre" (for VA-albums)
+            - original fallback
+            - configured fallback
             - None
+
+        A `(genre, label)` pair is returned, where `label` is a string used for
+        logging. For example, "keep + artist, whitelist" indicates that existing
+        genres were combined with new last.fm genres and whitelist filtering was
+        applied, while "artist, any" means only new last.fm genres are included
+        and the whitelist feature was disabled.
         """
 
-        # Shortcut to existing genre if not forcing.
-        if not self.config["force"] and self._is_allowed(obj.genre):
-            return obj.genre, "keep"
+        def _try_resolve_stage(stage_label: str, keep_genres, new_genres):
+            """Try to resolve genres for a given stage and log the result."""
+            resolved_genres = self._combine_resolve_and_log(
+                keep_genres, new_genres
+            )
+            if resolved_genres:
+                suffix = "whitelist" if self.whitelist else "any"
+                label = f"{stage_label}, {suffix}"
+                if keep_genres:
+                    label = f"keep + {label}"
+                return self._format_and_stringify(resolved_genres), label
+            return None
 
-        # Track genre (for Items only).
-        if isinstance(obj, library.Item):
-            if "track" in self.sources:
-                result = self.fetch_track_genre(obj)
-                if result:
-                    return result, "track"
+        keep_genres = []
+        new_genres = []
+        genres = self._get_existing_genres(obj)
 
-        # Album genre.
-        if "album" in self.sources:
-            result = self.fetch_album_genre(obj)
-            if result:
-                return result, "album"
-
-        # Artist (or album artist) genre.
-        if "artist" in self.sources:
-            result = None
+        if genres and not self.config["force"]:
+            # Without force pre-populated tags are returned as-is.
+            label = "keep any, no-force"
             if isinstance(obj, library.Item):
-                result = self.fetch_artist_genre(obj)
+                return obj.get("genre", with_album=False), label
+            return obj.get("genre"), label
+
+        if self.config["force"]:
+            # Force doesn't keep any unless keep_existing is set.
+            # Whitelist validation is handled in _resolve_genres.
+            if self.config["keep_existing"]:
+                keep_genres = [g.lower() for g in genres]
+
+        # Run through stages: track, album, artist,
+        # album artist, or most popular track genre.
+        if isinstance(obj, library.Item) and "track" in self.sources:
+            if new_genres := self.fetch_track_genre(obj):
+                if result := _try_resolve_stage(
+                    "track", keep_genres, new_genres
+                ):
+                    return result
+
+        if "album" in self.sources:
+            if new_genres := self.fetch_album_genre(obj):
+                if result := _try_resolve_stage(
+                    "album", keep_genres, new_genres
+                ):
+                    return result
+
+        if "artist" in self.sources:
+            new_genres = []
+            if isinstance(obj, library.Item):
+                new_genres = self.fetch_artist_genre(obj)
+                stage_label = "artist"
             elif obj.albumartist != config["va_name"].as_str():
-                result = self.fetch_album_artist_genre(obj)
+                new_genres = self.fetch_album_artist_genre(obj)
+                stage_label = "album artist"
             else:
                 # For "Various Artists", pick the most popular track genre.
                 item_genres = []
@@ -347,25 +428,36 @@ class LastGenrePlugin(plugins.BeetsPlugin):
                     if not item_genre:
                         item_genre = self.fetch_artist_genre(item)
                     if item_genre:
-                        item_genres.append(item_genre)
+                        item_genres += item_genre
                 if item_genres:
-                    result, _ = plurality(item_genres)
+                    most_popular, rank = plurality(item_genres)
+                    new_genres = [most_popular]
+                    stage_label = "most popular track"
+                    self._log.debug(
+                        'Most popular track genre "{}" ({}) for VA album.',
+                        most_popular,
+                        rank,
+                    )
 
-            if result:
-                return result, "artist"
+            if new_genres:
+                if result := _try_resolve_stage(
+                    stage_label, keep_genres, new_genres
+                ):
+                    return result
 
-        # Filter the existing genre.
-        if obj.genre:
-            result = self._resolve_genres([obj.genre])
-            if result:
-                return result, "original"
+        # Nothing found, leave original if configured and valid.
+        if obj.genre and self.config["keep_existing"]:
+            if not self.whitelist or self._is_valid(obj.genre.lower()):
+                return obj.genre, "original fallback"
 
-        # Fallback string.
-        fallback = self.config["fallback"].get()
-        if fallback:
+        # Return fallback string.
+        if fallback := self.config["fallback"].get():
             return fallback, "fallback"
 
-        return None, None
+        # No fallback configured.
+        return None, "fallback unconfigured"
+
+    # Beets plugin hooks and CLI.
 
     def commands(self):
         lastgenre_cmd = ui.Subcommand("lastgenre", help="fetch genres")
@@ -374,7 +466,28 @@ class LastGenrePlugin(plugins.BeetsPlugin):
             "--force",
             dest="force",
             action="store_true",
-            help="re-download genre when already present",
+            help="modify existing genres",
+        )
+        lastgenre_cmd.parser.add_option(
+            "-F",
+            "--no-force",
+            dest="force",
+            action="store_false",
+            help="don't modify existing genres",
+        )
+        lastgenre_cmd.parser.add_option(
+            "-k",
+            "--keep-existing",
+            dest="keep_existing",
+            action="store_true",
+            help="combine with existing genres when modifying",
+        )
+        lastgenre_cmd.parser.add_option(
+            "-K",
+            "--no-keep-existing",
+            dest="keep_existing",
+            action="store_false",
+            help="don't combine with existing genres when modifying",
         )
         lastgenre_cmd.parser.add_option(
             "-s",
@@ -395,7 +508,14 @@ class LastGenrePlugin(plugins.BeetsPlugin):
             "--albums",
             action="store_true",
             dest="album",
-            help="match albums instead of items",
+            help="match albums instead of items (default)",
+        )
+        lastgenre_cmd.parser.add_option(
+            "-d",
+            "--debug",
+            action="store_true",
+            dest="extended_debug",
+            help="extended last.fm debug logging",
         )
         lastgenre_cmd.parser.set_defaults(album=True)
 
@@ -405,12 +525,17 @@ class LastGenrePlugin(plugins.BeetsPlugin):
 
             if opts.album:
                 # Fetch genres for whole albums
-                for album in lib.albums(ui.decargs(args)):
+                for album in lib.albums(args):
                     album.genre, src = self._get_genre(album)
                     self._log.info(
-                        "genre for album {0} ({1}): {0.genre}", album, src
+                        'genre for album "{0.album}" ({1}): {0.genre}',
+                        album,
+                        src,
                     )
-                    album.store()
+                    if "track" in self.sources:
+                        album.store(inherit=False)
+                    else:
+                        album.store()
 
                     for item in album.items():
                         # If we're using track-level sources, also look up each
@@ -419,7 +544,7 @@ class LastGenrePlugin(plugins.BeetsPlugin):
                             item.genre, src = self._get_genre(item)
                             item.store()
                             self._log.info(
-                                "genre for track {0} ({1}): {0.genre}",
+                                'genre for track "{0.title}" ({1}): {0.genre}',
                                 item,
                                 src,
                             )
@@ -429,12 +554,12 @@ class LastGenrePlugin(plugins.BeetsPlugin):
             else:
                 # Just query singletons, i.e. items that are not part of
                 # an album
-                for item in lib.items(ui.decargs(args)):
+                for item in lib.items(args):
                     item.genre, src = self._get_genre(item)
-                    self._log.debug(
-                        "added last.fm item genre ({0}): {1}", src, item.genre
-                    )
                     item.store()
+                    self._log.info(
+                        "genre for track {0.title} ({1}): {0.genre}", item, src
+                    )
 
         lastgenre_cmd.func = lastgenre_func
         return [lastgenre_cmd]
@@ -445,23 +570,30 @@ class LastGenrePlugin(plugins.BeetsPlugin):
             album = task.album
             album.genre, src = self._get_genre(album)
             self._log.debug(
-                "added last.fm album genre ({0}): {1}", src, album.genre
+                'genre for album "{0.album}" ({1}): {0.genre}', album, src
             )
-            album.store()
 
+            # If we're using track-level sources, store the album genre only,
+            # then also look up individual track genres.
             if "track" in self.sources:
+                album.store(inherit=False)
                 for item in album.items():
                     item.genre, src = self._get_genre(item)
                     self._log.debug(
-                        "added last.fm item genre ({0}): {1}", src, item.genre
+                        'genre for track "{0.title}" ({1}): {0.genre}',
+                        item,
+                        src,
                     )
                     item.store()
+            # Store the album genre and inherit to tracks.
+            else:
+                album.store()
 
         else:
             item = task.item
             item.genre, src = self._get_genre(item)
             self._log.debug(
-                "added last.fm item genre ({0}): {1}", src, item.genre
+                'genre for track "{0.title}" ({1}): {0.genre}', item, src
             )
             item.store()
 
@@ -483,12 +615,12 @@ class LastGenrePlugin(plugins.BeetsPlugin):
         try:
             res = obj.get_top_tags()
         except PYLAST_EXCEPTIONS as exc:
-            self._log.debug("last.fm error: {0}", exc)
+            self._log.debug("last.fm error: {}", exc)
             return []
         except Exception as exc:
             # Isolate bugs in pylast.
             self._log.debug("{}", traceback.format_exc())
-            self._log.error("error in pylast library: {0}", exc)
+            self._log.error("error in pylast library: {}", exc)
             return []
 
         # Filter by weight (optionally).
