@@ -17,34 +17,71 @@
 Spotify playlist construction.
 """
 
+from __future__ import annotations
+
 import base64
 import collections
 import json
 import re
 import time
 import webbrowser
+from typing import TYPE_CHECKING, Any, Literal, Sequence, Union
 
 import confuse
 import requests
-import unidecode
 
 from beets import ui
 from beets.autotag.hooks import AlbumInfo, TrackInfo
 from beets.dbcore import types
-from beets.library import DateType
-from beets.plugins import BeetsPlugin, MetadataSourcePlugin
-from beets.util.id_extractors import spotify_id_regex
+from beets.library import Library
+from beets.metadata_plugins import (
+    IDResponse,
+    SearchApiMetadataSourcePlugin,
+    SearchFilter,
+)
+
+if TYPE_CHECKING:
+    from beets.library import Library
+    from beetsplug._typing import JSONDict
 
 DEFAULT_WAITING_TIME = 5
 
 
-class SpotifyAPIError(Exception):
+class SearchResponseAlbums(IDResponse):
+    """A response returned by the Spotify API.
+
+    We only use items and disregard the pagination information.
+    i.e. res["albums"]["items"][0].
+
+    There are more fields in the response, but we only type
+    the ones we currently use.
+
+    see https://developer.spotify.com/documentation/web-api/reference/search
+    """
+
+    album_type: str
+    available_markets: Sequence[str]
+    name: str
+
+
+class SearchResponseTracks(IDResponse):
+    """A track response returned by the Spotify API."""
+
+    album: SearchResponseAlbums
+    available_markets: Sequence[str]
+    popularity: int
+    name: str
+
+
+class APIError(Exception):
     pass
 
 
-class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
-    data_source = "Spotify"
-
+class SpotifyPlugin(
+    SearchApiMetadataSourcePlugin[
+        Union[SearchResponseAlbums, SearchResponseTracks]
+    ]
+):
     item_types = {
         "spotify_track_popularity": types.INTEGER,
         "spotify_acousticness": types.FLOAT,
@@ -59,7 +96,7 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
         "spotify_tempo": types.FLOAT,
         "spotify_time_signature": types.INTEGER,
         "spotify_valence": types.FLOAT,
-        "spotify_updated": DateType(),
+        "spotify_updated": types.DATE,
     }
 
     # Base URLs for the Spotify API
@@ -71,8 +108,6 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
     track_url = "https://api.spotify.com/v1/tracks/"
     audio_features_url = "https://api.spotify.com/v1/audio-features/"
 
-    id_regex = spotify_id_regex
-
     spotify_audio_features = {
         "acousticness": "spotify_acousticness",
         "danceability": "spotify_danceability",
@@ -105,75 +140,81 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
                 "tokenfile": "spotify_token.json",
             }
         )
+        self.config["client_id"].redact = True
         self.config["client_secret"].redact = True
 
-        self.tokenfile = self.config["tokenfile"].get(
-            confuse.Filename(in_app_dir=True)
-        )  # Path to the JSON file for storing the OAuth access token.
         self.setup()
 
     def setup(self):
         """Retrieve previously saved OAuth token or generate a new one."""
+
         try:
-            with open(self.tokenfile) as f:
+            with open(self._tokenfile()) as f:
                 token_data = json.load(f)
         except OSError:
             self._authenticate()
         else:
             self.access_token = token_data["access_token"]
 
-    def _authenticate(self):
+    def _tokenfile(self) -> str:
+        """Get the path to the JSON file for storing the OAuth token."""
+        return self.config["tokenfile"].get(confuse.Filename(in_app_dir=True))
+
+    def _authenticate(self) -> None:
         """Request an access token via the Client Credentials Flow:
         https://developer.spotify.com/documentation/general/guides/authorization-guide/#client-credentials-flow
         """
+        c_id: str = self.config["client_id"].as_str()
+        c_secret: str = self.config["client_secret"].as_str()
+
         headers = {
-            "Authorization": "Basic {}".format(
-                base64.b64encode(
-                    ":".join(
-                        self.config[k].as_str()
-                        for k in ("client_id", "client_secret")
-                    ).encode()
-                ).decode()
+            "Authorization": (
+                "Basic"
+                f" {base64.b64encode(f'{c_id}:{c_secret}'.encode()).decode()}"
             )
         }
         response = requests.post(
             self.oauth_token_url,
             data={"grant_type": "client_credentials"},
             headers=headers,
+            timeout=10,
         )
         try:
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
             raise ui.UserError(
-                "Spotify authorization failed: {}\n{}".format(e, response.text)
+                f"Spotify authorization failed: {e}\n{response.text}"
             )
         self.access_token = response.json()["access_token"]
 
         # Save the token for later use.
-        self._log.debug(
-            "{} access token: {}", self.data_source, self.access_token
-        )
-        with open(self.tokenfile, "w") as f:
+        self._log.debug("{0.data_source} access token: {0.access_token}", self)
+        with open(self._tokenfile(), "w") as f:
             json.dump({"access_token": self.access_token}, f)
 
     def _handle_response(
-        self, request_type, url, params=None, retry_count=0, max_retries=3
-    ):
+        self,
+        method: Literal["get", "post", "put", "delete"],
+        url: str,
+        params: Any = None,
+        retry_count: int = 0,
+        max_retries: int = 3,
+    ) -> JSONDict:
         """Send a request, reauthenticating if necessary.
 
-        :param request_type: Type of :class:`Request` constructor,
-            e.g. ``requests.get``, ``requests.post``, etc.
-        :type request_type: function
+        :param method: HTTP method to use for the request.
         :param url: URL for the new :class:`Request` object.
-        :type url: str
         :param params: (optional) list of tuples or bytes to send
             in the query string for the :class:`Request`.
         :type params: dict
-        :return: JSON data for the class:`Response <Response>` object.
-        :rtype: dict
         """
+
+        if retry_count > max_retries:
+            raise APIError("Maximum retries reached.")
+
         try:
-            response = request_type(
+            response = requests.request(
+                method,
                 url,
                 headers={"Authorization": f"Bearer {self.access_token}"},
                 params=params,
@@ -183,56 +224,61 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
             return response.json()
         except requests.exceptions.ReadTimeout:
             self._log.error("ReadTimeout.")
-            raise SpotifyAPIError("Request timed out.")
+            raise APIError("Request timed out.")
         except requests.exceptions.ConnectionError as e:
-            self._log.error(f"Network error: {e}")
-            raise SpotifyAPIError("Network error.")
+            self._log.error("Network error: {}", e)
+            raise APIError("Network error.")
         except requests.exceptions.RequestException as e:
+            if e.response is None:
+                self._log.error("Request failed: {}", e)
+                raise APIError("Request failed.")
             if e.response.status_code == 401:
                 self._log.debug(
-                    f"{self.data_source} access token has expired. "
-                    f"Reauthenticating."
+                    "{.data_source} access token has expired. Reauthenticating.",
+                    self,
                 )
                 self._authenticate()
-                return self._handle_response(request_type, url, params=params)
+                return self._handle_response(
+                    method,
+                    url,
+                    params=params,
+                    retry_count=retry_count + 1,
+                )
             elif e.response.status_code == 404:
-                raise SpotifyAPIError(
+                raise APIError(
                     f"API Error: {e.response.status_code}\n"
                     f"URL: {url}\nparams: {params}"
                 )
             elif e.response.status_code == 429:
-                if retry_count >= max_retries:
-                    raise SpotifyAPIError("Maximum retries reached.")
-                seconds = response.headers.get(
+                seconds = e.response.headers.get(
                     "Retry-After", DEFAULT_WAITING_TIME
                 )
                 self._log.debug(
-                    f"Too many API requests. Retrying after "
-                    f"{seconds} seconds."
+                    "Too many API requests. Retrying after {} seconds.", seconds
                 )
                 time.sleep(int(seconds) + 1)
                 return self._handle_response(
-                    request_type,
+                    method,
                     url,
                     params=params,
                     retry_count=retry_count + 1,
                 )
             elif e.response.status_code == 503:
                 self._log.error("Service Unavailable.")
-                raise SpotifyAPIError("Service Unavailable.")
+                raise APIError("Service Unavailable.")
             elif e.response.status_code == 502:
                 self._log.error("Bad Gateway.")
-                raise SpotifyAPIError("Bad Gateway.")
+                raise APIError("Bad Gateway.")
             elif e.response is not None:
-                raise SpotifyAPIError(
+                raise APIError(
                     f"{self.data_source} API error:\n{e.response.text}\n"
                     f"URL:\n{url}\nparams:\n{params}"
                 )
             else:
-                self._log.error(f"Request failed. Error: {e}")
-                raise SpotifyAPIError("Request failed.")
+                self._log.error("Request failed. Error: {}", e)
+                raise APIError("Request failed.")
 
-    def album_for_id(self, album_id):
+    def album_for_id(self, album_id: str) -> AlbumInfo | None:
         """Fetch an album by its Spotify ID or URL and return an
         AlbumInfo object or None if the album is not found.
 
@@ -241,12 +287,11 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
         :return: AlbumInfo object for album
         :rtype: beets.autotag.hooks.AlbumInfo or None
         """
-        spotify_id = self._get_id("album", album_id, self.id_regex)
-        if spotify_id is None:
+        if not (spotify_id := self._extract_id(album_id)):
             return None
 
         album_data = self._handle_response(
-            requests.get, self.album_url + spotify_id
+            "get", f"{self.album_url}{spotify_id}"
         )
         if album_data["name"] == "":
             self._log.debug("Album removed from Spotify: {}", album_id)
@@ -270,21 +315,17 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
         else:
             raise ui.UserError(
                 "Invalid `release_date_precision` returned "
-                "by {} API: '{}'".format(
-                    self.data_source, release_date_precision
-                )
+                f"by {self.data_source} API: '{release_date_precision}'"
             )
 
         tracks_data = album_data["tracks"]
         tracks_items = tracks_data["items"]
         while tracks_data["next"]:
-            tracks_data = self._handle_response(
-                requests.get, tracks_data["next"]
-            )
+            tracks_data = self._handle_response("get", tracks_data["next"])
             tracks_items.extend(tracks_data["items"])
 
         tracks = []
-        medium_totals = collections.defaultdict(int)
+        medium_totals: dict[int | None, int] = collections.defaultdict(int)
         for i, track_data in enumerate(tracks_items, start=1):
             track = self._get_track(track_data)
             track.index = i
@@ -308,19 +349,17 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
             month=month,
             day=day,
             label=album_data["label"],
-            mediums=max(medium_totals.keys()),
+            mediums=max(filter(None, medium_totals.keys())),
             data_source=self.data_source,
             data_url=album_data["external_urls"]["spotify"],
         )
 
-    def _get_track(self, track_data):
+    def _get_track(self, track_data: JSONDict) -> TrackInfo:
         """Convert a Spotify track object dict to a TrackInfo object.
 
         :param track_data: Simplified track object
             (https://developer.spotify.com/documentation/web-api/reference/object-model/#track-object-simplified)
-        :type track_data: dict
         :return: TrackInfo object for track
-        :rtype: beets.autotag.hooks.TrackInfo
         """
         artist, artist_id = self.get_artist(track_data["artists"])
 
@@ -345,33 +384,31 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
             data_url=track_data["external_urls"]["spotify"],
         )
 
-    def track_for_id(self, track_id=None, track_data=None):
-        """Fetch a track by its Spotify ID or URL and return a
-        TrackInfo object or None if the track is not found.
+    def track_for_id(self, track_id: str) -> None | TrackInfo:
+        """Fetch a track by its Spotify ID or URL.
 
-        :param track_id: (Optional) Spotify ID or URL for the track. Either
-            ``track_id`` or ``track_data`` must be provided.
-        :type track_id: str
-        :param track_data: (Optional) Simplified track object dict. May be
-            provided instead of ``track_id`` to avoid unnecessary API calls.
-        :type track_data: dict
-        :return: TrackInfo object for track
-        :rtype: beets.autotag.hooks.TrackInfo or None
+        Returns a TrackInfo object or None if the track is not found.
         """
-        if track_data is None:
-            spotify_id = self._get_id("track", track_id, self.id_regex)
-            if spotify_id is None:
-                return None
-            track_data = self._handle_response(
-                requests.get, self.track_url + spotify_id
+
+        if not (spotify_id := self._extract_id(track_id)):
+            self._log.debug("Invalid Spotify ID: {}", track_id)
+            return None
+
+        if not (
+            track_data := self._handle_response(
+                "get", f"{self.track_url}{spotify_id}"
             )
+        ):
+            self._log.debug("Track not found: {}", track_id)
+            return None
+
         track = self._get_track(track_data)
 
         # Get album's tracks to set `track.index` (position on the entire
         # release) and `track.medium_total` (total number of tracks on
         # the track's disc).
         album_data = self._handle_response(
-            requests.get, self.album_url + track_data["album"]["id"]
+            "get", f"{self.album_url}{track_data['album']['id']}"
         )
         medium_total = 0
         for i, track_data in enumerate(album_data["tracks"]["items"], start=1):
@@ -382,71 +419,53 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
         track.medium_total = medium_total
         return track
 
-    @staticmethod
-    def _construct_search_query(filters=None, keywords=""):
-        """Construct a query string with the specified filters and keywords to
-        be provided to the Spotify Search API
-        (https://developer.spotify.com/documentation/web-api/reference/search/search/#writing-a-query---guidelines).
-
-        :param filters: (Optional) Field filters to apply.
-        :type filters: dict
-        :param keywords: (Optional) Query keywords to use.
-        :type keywords: str
-        :return: Query string to be provided to the Search API.
-        :rtype: str
-        """
-        query_components = [
-            keywords,
-            " ".join(":".join((k, v)) for k, v in filters.items()),
-        ]
-        query = " ".join([q for q in query_components if q])
-        if not isinstance(query, str):
-            query = query.decode("utf8")
-        return unidecode.unidecode(query)
-
-    def _search_api(self, query_type, filters=None, keywords=""):
-        """Query the Spotify Search API for the specified ``keywords``,
+    def _search_api(
+        self,
+        query_type: Literal["album", "track"],
+        filters: SearchFilter,
+        query_string: str = "",
+    ) -> Sequence[SearchResponseAlbums | SearchResponseTracks]:
+        """Query the Spotify Search API for the specified ``query_string``,
         applying the provided ``filters``.
 
         :param query_type: Item type to search across. Valid types are:
             'album', 'artist', 'playlist', and 'track'.
-        :type query_type: str
-        :param filters: (Optional) Field filters to apply.
-        :type filters: dict
-        :param keywords: (Optional) Query keywords to use.
-        :type keywords: str
-        :return: JSON data for the class:`Response <Response>` object or None
-            if no search results are returned.
-        :rtype: dict or None
+        :param filters: Field filters to apply.
+        :param query_string: Additional query to include in the search.
         """
-        query = self._construct_search_query(keywords=keywords, filters=filters)
-        if not query:
-            return None
-        self._log.debug(f"Searching {self.data_source} for '{query}'")
+        query = self._construct_search_query(
+            filters=filters, query_string=query_string
+        )
+
+        self._log.debug("Searching {.data_source} for '{}'", self, query)
         try:
             response = self._handle_response(
-                requests.get,
+                "get",
                 self.search_url,
-                params={"q": query, "type": query_type},
+                params={
+                    "q": query,
+                    "type": query_type,
+                    "limit": self.config["search_limit"].get(),
+                },
             )
-        except SpotifyAPIError as e:
+        except APIError as e:
             self._log.debug("Spotify API error: {}", e)
-            return []
-        response_data = response.get(query_type + "s", {}).get("items", [])
+            return ()
+        response_data = response.get(f"{query_type}s", {}).get("items", [])
         self._log.debug(
-            "Found {} result(s) from {} for '{}'",
+            "Found {} result(s) from {.data_source} for '{}'",
             len(response_data),
-            self.data_source,
+            self,
             query,
         )
         return response_data
 
-    def commands(self):
+    def commands(self) -> list[ui.Subcommand]:
         # autotagger import command
         def queries(lib, opts, args):
             success = self._parse_opts(opts)
             if success:
-                results = self._match_library_tracks(lib, ui.decargs(args))
+                results = self._match_library_tracks(lib, args)
                 self._output_match_results(results)
 
         spotify_cmd = ui.Subcommand(
@@ -456,17 +475,17 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
             "-m",
             "--mode",
             action="store",
-            help='"open" to open {} with playlist, '
-            '"list" to print (default)'.format(self.data_source),
+            help=(
+                f'"open" to open {self.data_source} with playlist, '
+                '"list" to print (default)'
+            ),
         )
         spotify_cmd.parser.add_option(
             "-f",
             "--show-failures",
             action="store_true",
             dest="show_failures",
-            help="list tracks that did not match a {} ID".format(
-                self.data_source
-            ),
+            help=f"list tracks that did not match a {self.data_source} ID",
         )
         spotify_cmd.func = queries
 
@@ -484,7 +503,7 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
         )
 
         def func(lib, opts, args):
-            items = lib.items(ui.decargs(args))
+            items = lib.items(args)
             self._fetch_info(items, ui.should_write(), opts.force_refetch)
 
         sync_cmd.func = func
@@ -499,24 +518,21 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
 
         if self.config["mode"].get() not in ["list", "open"]:
             self._log.warning(
-                "{0} is not a valid mode", self.config["mode"].get()
+                "{} is not a valid mode", self.config["mode"].get()
             )
             return False
 
         self.opts = opts
         return True
 
-    def _match_library_tracks(self, library, keywords):
+    def _match_library_tracks(self, library: Library, keywords: str):
         """Get a list of simplified track object dicts for library tracks
         matching the specified ``keywords``.
 
         :param library: beets library object to query.
-        :type library: beets.library.Library
         :param keywords: Query to match library items against.
-        :type keywords: str
         :return: List of simplified track object dicts for library items
             matching the specified query.
-        :rtype: list[dict]
         """
         results = []
         failures = []
@@ -525,8 +541,8 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
 
         if not items:
             self._log.debug(
-                "Your beets query returned no items, skipping {}.",
-                self.data_source,
+                "Your beets query returned no items, skipping {.data_source}.",
+                self,
             )
             return
 
@@ -550,22 +566,25 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
             # Custom values can be passed in the config (just in case)
             artist = item[self.config["artist_field"].get()]
             album = item[self.config["album_field"].get()]
-            keywords = item[self.config["track_field"].get()]
+            query_string = item[self.config["track_field"].get()]
 
             # Query the Web API for each track, look for the items' JSON data
-            query_filters = {"artist": artist, "album": album}
+            query_filters: SearchFilter = {"artist": artist, "album": album}
             response_data_tracks = self._search_api(
-                query_type="track", keywords=keywords, filters=query_filters
+                query_type="track",
+                query_string=query_string,
+                filters=query_filters,
             )
             if not response_data_tracks:
                 query = self._construct_search_query(
-                    keywords=keywords, filters=query_filters
+                    query_string=query_string, filters=query_filters
                 )
+
                 failures.append(query)
                 continue
 
             # Apply market filter if requested
-            region_filter = self.config["region_filter"].get()
+            region_filter: str = self.config["region_filter"].get()
             if region_filter:
                 response_data_tracks = [
                     track_data
@@ -578,8 +597,8 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
                 or self.config["tiebreak"].get() == "first"
             ):
                 self._log.debug(
-                    "{} track(s) found, count: {}",
-                    self.data_source,
+                    "{.data_source} track(s) found, count: {}",
+                    self,
                     len(response_data_tracks),
                 )
                 chosen_result = response_data_tracks[0]
@@ -590,7 +609,11 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
                     len(response_data_tracks),
                 )
                 chosen_result = max(
-                    response_data_tracks, key=lambda x: x["popularity"]
+                    response_data_tracks,
+                    key=lambda x: x[
+                        # We are sure this is a track response!
+                        "popularity"  # type: ignore[typeddict-item]
+                    ],
                 )
             results.append(chosen_result)
 
@@ -598,19 +621,19 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
         if failure_count > 0:
             if self.config["show_failures"].get():
                 self._log.info(
-                    "{} track(s) did not match a {} ID:",
+                    "{} track(s) did not match a {.data_source} ID:",
                     failure_count,
-                    self.data_source,
+                    self,
                 )
                 for track in failures:
                     self._log.info("track: {}", track)
                 self._log.info("")
             else:
                 self._log.warning(
-                    "{} track(s) did not match a {} ID:\n"
+                    "{} track(s) did not match a {.data_source} ID:\n"
                     "use --show-failures to display",
                     failure_count,
-                    self.data_source,
+                    self,
                 )
 
         return results
@@ -627,20 +650,18 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
             spotify_ids = [track_data["id"] for track_data in results]
             if self.config["mode"].get() == "open":
                 self._log.info(
-                    "Attempting to open {} with playlist".format(
-                        self.data_source
-                    )
+                    "Attempting to open {.data_source} with playlist", self
                 )
-                spotify_url = "spotify:trackset:Playlist:" + ",".join(
-                    spotify_ids
+                spotify_url = (
+                    f"spotify:trackset:Playlist:{','.join(spotify_ids)}"
                 )
                 webbrowser.open(spotify_url)
             else:
                 for spotify_id in spotify_ids:
-                    print(self.open_track_url + spotify_id)
+                    print(f"{self.open_track_url}{spotify_id}")
         else:
             self._log.warning(
-                f"No {self.data_source} tracks found from beets query"
+                "No {.data_source} tracks found from beets query", self
             )
 
     def _fetch_info(self, items, write, force):
@@ -683,29 +704,29 @@ class SpotifyPlugin(MetadataSourcePlugin, BeetsPlugin):
             if write:
                 item.try_write()
 
-    def track_info(self, track_id=None):
+    def track_info(self, track_id: str):
         """Fetch a track's popularity and external IDs using its Spotify ID."""
-        track_data = self._handle_response(
-            requests.get, self.track_url + track_id
-        )
+        track_data = self._handle_response("get", f"{self.track_url}{track_id}")
+        external_ids = track_data.get("external_ids", {})
+        popularity = track_data.get("popularity")
         self._log.debug(
             "track_popularity: {} and track_isrc: {}",
-            track_data.get("popularity"),
-            track_data.get("external_ids").get("isrc"),
+            popularity,
+            external_ids.get("isrc"),
         )
         return (
-            track_data.get("popularity"),
-            track_data.get("external_ids").get("isrc"),
-            track_data.get("external_ids").get("ean"),
-            track_data.get("external_ids").get("upc"),
+            popularity,
+            external_ids.get("isrc"),
+            external_ids.get("ean"),
+            external_ids.get("upc"),
         )
 
-    def track_audio_features(self, track_id=None):
+    def track_audio_features(self, track_id: str):
         """Fetch track audio features by its Spotify ID."""
         try:
             return self._handle_response(
-                requests.get, self.audio_features_url + track_id
+                "get", f"{self.audio_features_url}{track_id}"
             )
-        except SpotifyAPIError as e:
+        except APIError as e:
             self._log.debug("Spotify API error: {}", e)
             return None
