@@ -12,17 +12,16 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-""" Clears tag fields in media files."""
-
+"""Clears tag fields in media files."""
 
 import re
 
 import confuse
 from mediafile import MediaFile
 
-from beets.importer import action
+from beets.importer import Action
 from beets.plugins import BeetsPlugin
-from beets.ui import Subcommand, decargs, input_yn
+from beets.ui import Subcommand, input_yn
 
 __author__ = "baobab@heresiarch.info"
 
@@ -76,11 +75,11 @@ class ZeroPlugin(BeetsPlugin):
         zero_command = Subcommand("zero", help="set fields to null")
 
         def zero_fields(lib, opts, args):
-            if not decargs(args) and not input_yn(
+            if not args and not input_yn(
                 "Remove fields for all items? (Y/n)", True
             ):
                 return
-            for item in lib.items(decargs(args)):
+            for item in lib.items(args):
                 self.process_item(item)
 
         zero_command.func = zero_fields
@@ -91,10 +90,10 @@ class ZeroPlugin(BeetsPlugin):
         Do some sanity checks then compile the regexes.
         """
         if field not in MediaFile.fields():
-            self._log.error("invalid field: {0}", field)
+            self._log.error("invalid field: {}", field)
         elif field in ("id", "path", "album_id"):
             self._log.warning(
-                "field '{0}' ignored, zeroing " "it would be dangerous", field
+                "field '{}' ignored, zeroing it would be dangerous", field
             )
         else:
             try:
@@ -106,7 +105,7 @@ class ZeroPlugin(BeetsPlugin):
                 self.fields_to_progs[field] = []
 
     def import_task_choice_event(self, session, task):
-        if task.choice_flag == action.ASIS and not self.warned:
+        if task.choice_flag == Action.ASIS and not self.warned:
             self._log.warning('cannot zero in "as-is" mode')
             self.warned = True
         # TODO request write in as-is mode
@@ -138,7 +137,7 @@ class ZeroPlugin(BeetsPlugin):
 
             if match:
                 fields_set = True
-                self._log.debug("{0}: {1} -> None", field, value)
+                self._log.debug("{}: {} -> None", field, value)
                 tags[field] = None
                 if self.config["update_database"]:
                     item[field] = None
